@@ -205,21 +205,18 @@ set_view (GtkChamplainEmbed *embed,
   stage = gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->clutter_embed));
 
   if (priv->view != NULL)
-    clutter_container_remove_actor (CLUTTER_CONTAINER (stage), CLUTTER_ACTOR (priv->view));
+    clutter_actor_remove_child (stage, CLUTTER_ACTOR (priv->view));
 
   priv->view = view;
   clutter_actor_set_size (CLUTTER_ACTOR (priv->view), priv->width, priv->height);
 
-  clutter_container_add_actor (CLUTTER_CONTAINER (stage), CLUTTER_ACTOR (priv->view));
+  clutter_actor_add_child (stage, CLUTTER_ACTOR (priv->view));
 }
 
 
 static void
 gtk_champlain_embed_init (GtkChamplainEmbed *embed)
 {
-  ClutterColor stage_color = { 0x34, 0x39, 0x39, 0xff };
-  ClutterActor *stage;
-
   GtkChamplainEmbedPrivate *priv = GET_PRIVATE (embed);
 
   embed->priv = priv;
@@ -249,10 +246,6 @@ gtk_champlain_embed_init (GtkChamplainEmbed *embed)
   priv->view = NULL;
   set_view (embed, CHAMPLAIN_VIEW (champlain_view_new ()));
 
-  /* Setup stage */
-  stage = gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->clutter_embed));
-  clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
-
   gtk_container_add (GTK_CONTAINER (embed), priv->clutter_embed);
 }
 
@@ -294,9 +287,23 @@ view_realize_cb (GtkWidget *widget,
   style = gtk_widget_get_style (widget);
 
   gdk_to_clutter_color (&style->text[GTK_STATE_SELECTED], &color);
+  if (color.alpha == 0 && color.red == 0 && color.green == 0 && color.blue == 0)
+    {
+      color.red = 255;
+      color.green = 255;
+      color.blue = 255;
+    }
   champlain_marker_set_selection_text_color (&color);
 
   gdk_to_clutter_color (&style->bg[GTK_STATE_SELECTED], &color);
+  if (color.alpha == 0)
+    color.alpha = 255;
+  if (color.red == 0 && color.green == 0 && color.blue == 0)
+    {
+      color.red = 75;
+      color.green = 105;
+      color.blue = 131;
+    }
   champlain_marker_set_selection_color (&color);
 #else
   GtkStyleContext *style;
@@ -307,10 +314,24 @@ view_realize_cb (GtkWidget *widget,
   
   gtk_style_context_get_color (style, GTK_STATE_FLAG_SELECTED, &gdk_rgba_color);
   gdk_rgba_to_clutter_color (&gdk_rgba_color, &color);
+  if (color.alpha == 0 && color.red == 0 && color.green == 0 && color.blue == 0)
+    {
+      color.red = 255;
+      color.green = 255;
+      color.blue = 255;
+    }
   champlain_marker_set_selection_text_color (&color);
 
   gtk_style_context_get_background_color (style, GTK_STATE_FLAG_SELECTED, &gdk_rgba_color);
   gdk_rgba_to_clutter_color (&gdk_rgba_color, &color);
+  if (color.alpha == 0)
+    color.alpha = 255;
+  if (color.red == 0 && color.green == 0 && color.blue == 0)
+    {
+      color.red = 75;
+      color.green = 105;
+      color.blue = 131;
+    }
   champlain_marker_set_selection_color (&color);
 #endif
 
