@@ -150,7 +150,7 @@ champlain_map_source_factory_init (ChamplainMapSourceFactory *factory)
   desc = champlain_map_source_desc_new_full (
         CHAMPLAIN_MAP_SOURCE_OSM_MAPNIK,
         "OpenStreetMap Mapnik",
-        "Map data is CC-BY-SA 2.0 OpenStreetMap contributors",
+        "Map Data ODBL OpenStreetMap Contributors, Map Imagery CC-BY-SA 2.0 OpenStreetMap",
         "http://creativecommons.org/licenses/by-sa/2.0/",
         0,
         18,
@@ -262,6 +262,76 @@ champlain_map_source_factory_init (ChamplainMapSourceFactory *factory)
         NULL);
   champlain_map_source_factory_register (factory, desc);
 
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OWM_CLOUDS,
+        "OpenWeatherMap cloud layer",
+        "Map data is CC-BY-SA 2.0 OpenWeatherMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        18,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://tile.openweathermap.org/map/clouds/#Z#/#X#/#Y#.png",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
+
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OWM_WIND,
+        "OpenWeatherMap wind layer",
+        "Map data is CC-BY-SA 2.0 OpenWeatherMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        18,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://tile.openweathermap.org/map/wind/#Z#/#X#/#Y#.png",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
+
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OWM_TEMPERATURE,
+        "OpenWeatherMap temperature layer",
+        "Map data is CC-BY-SA 2.0 OpenWeatherMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        18,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://tile.openweathermap.org/map/temp/#Z#/#X#/#Y#.png",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
+
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OWM_PRECIPITATION,
+        "OpenWeatherMap precipitation layer",
+        "Map data is CC-BY-SA 2.0 OpenWeatherMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        18,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://tile.openweathermap.org/map/precipitation/#Z#/#X#/#Y#.png",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
+
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OWM_PRESSURE,
+        "OpenWeatherMap sea level pressure layer",
+        "Map data is CC-BY-SA 2.0 OpenWeatherMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        18,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://tile.openweathermap.org/map/pressure/#Z#/#X#/#Y#.png",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
+
 #ifdef CHAMPLAIN_HAS_MEMPHIS
   desc = champlain_map_source_desc_new_full (
         CHAMPLAIN_MAP_SOURCE_MEMPHIS_LOCAL,
@@ -413,6 +483,40 @@ champlain_map_source_factory_create_cached_source (ChamplainMapSourceFactory *fa
 }
 
 
+/**
+ * champlain_map_source_factory_create_memcached_source:
+ * @factory: the Factory
+ * @id: the wanted map source id
+ *
+ * Creates a memory cached map source.
+ *
+ * Returns: (transfer none): a ready to use #ChamplainMapSourceChain consisting of
+ * #ChamplainMemoryCache and #ChamplainMapSource matching the given name
+ *
+ * Since: 0.12.5
+ */
+ChamplainMapSource *
+champlain_map_source_factory_create_memcached_source (ChamplainMapSourceFactory *factory,
+    const gchar *id)
+{
+  ChamplainMapSourceChain *source_chain;
+  ChamplainMapSource *tile_source;
+  ChamplainMapSource *memory_cache;
+  ChamplainRenderer *renderer;
+
+  tile_source = champlain_map_source_factory_create (factory, id);
+
+  renderer = CHAMPLAIN_RENDERER (champlain_image_renderer_new ());
+  memory_cache = CHAMPLAIN_MAP_SOURCE (champlain_memory_cache_new_full (100, renderer));
+
+  source_chain = champlain_map_source_chain_new ();
+  champlain_map_source_chain_push (source_chain, tile_source);
+  champlain_map_source_chain_push (source_chain, memory_cache);
+
+  return CHAMPLAIN_MAP_SOURCE (source_chain);
+}
+
+
 /**
  * champlain_map_source_factory_create_error_source:
  * @factory: the Factory
